/*
 * Copyright (C) 2021 Thomas Van Acker
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

package be.bitscuit.calendar.Activities;

import androidx.appcompat.app.ActionBar;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;

import android.app.AlertDialog;
import android.app.DatePickerDialog;
import android.app.TimePickerDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.inputmethod.EditorInfo;
import android.view.inputmethod.InputMethodManager;
import android.widget.CompoundButton;
import android.widget.DatePicker;
import android.widget.EditText;
import android.widget.Switch;
import android.widget.TextView;
import android.widget.TimePicker;

import java.util.Calendar;

import be.bitscuit.calendar.R;
import be.bitscuit.calendar.Util.TaskData;
import be.bitscuit.calendar.Util.TaskUtil;
import be.bitscuit.calendar.Util.Util;

public class EditActivity extends AppCompatActivity {

    private EditText editTitle, editDateStart, editDateEnd, editTimeStart, editTimeEnd, editNotify,
            editRepeatEvery, editRepeatTimes, editDescription;
    private Switch switchRepeat;
    private TextView labelRepeatEvery, labelRepeatTimes, labelRepeat;

    private TaskData taskData;
    private TaskUtil taskUtil;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_edit);

        int taskID = getIntent().getIntExtra("TASK_ID", 0);

        //Init vars
        taskUtil = new TaskUtil(this);
        taskData = taskID == 0 ? new TaskData() : taskUtil.getByTaskID(taskID);

        if(taskID == 0) {//Only set time when creating new task
            long taskTimeStart = getIntent().getLongExtra("TASK_TIME_START", System.currentTimeMillis());
            taskData.setTimeStart(taskTimeStart);
            taskData.setTimeEnd(taskTimeStart + 3600000);
            taskData.setTimeNotify(0);//Default no reminder
        }

        //Set toolbar
        Toolbar toolbar = findViewById(R.id.edit_toolbar);
        setSupportActionBar(toolbar);
        setTitle( (taskID == 0?"Add":"Edit") + " Appointment");
        ActionBar actionBar = getSupportActionBar();
        actionBar.setDisplayHomeAsUpEnabled(true);

        //Init views
        editTitle = findViewById(R.id.edit_edit_title);
        editDescription = findViewById(R.id.edit_edit_description);
        editDateStart = findViewById(R.id.edit_edit_date_start);
        editDateEnd = findViewById(R.id.edit_edit_date_end);
        editTimeStart = findViewById(R.id.edit_edit_time_start);
        editTimeEnd = findViewById(R.id.edit_edit_time_end);
        editNotify = findViewById(R.id.edit_edit_notify);
        editRepeatEvery = findViewById(R.id.edit_edit_repeat_every);
        editRepeatTimes = findViewById(R.id.edit_edit_repeat_times);
        switchRepeat = findViewById(R.id.edit_switch_batch);
        labelRepeatEvery = findViewById(R.id.edit_label_repeat_every);
        labelRepeatTimes = findViewById(R.id.edit_label_repeat_times);
        labelRepeat = findViewById(R.id.edit_label_batch);

        //Edit view properties
        editTitle.setOnEditorActionListener(new TextView.OnEditorActionListener() {
            @Override
            public boolean onEditorAction(TextView textView, int i, KeyEvent keyEvent) {
                if(i == EditorInfo.IME_ACTION_DONE) {
                    InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
                    imm.hideSoftInputFromWindow(editTitle.getWindowToken(), 0);
                    editTitle.clearFocus();
                    return true;
                }
                return false;
            }
        });
        editDescription.setOnEditorActionListener(new TextView.OnEditorActionListener() {
            @Override
            public boolean onEditorAction(TextView textView, int i, KeyEvent keyEvent) {
                if(i == EditorInfo.IME_ACTION_DONE) {
                    InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
                    imm.hideSoftInputFromWindow(editDescription.getWindowToken(), 0);
                    editDescription.clearFocus();
                    return true;
                }
                return false;
            }
        });
        editDateStart.setFocusable(false);
        editDateStart.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                Calendar calendar = Calendar.getInstance();
                calendar.setTimeInMillis(taskData.getTimeStart());
                new DatePickerDialog(view.getContext(), new DatePickerDialog.OnDateSetListener() {
                    @Override
                    public void onDateSet(DatePicker datePicker, int i, int i1, int i2) {
                        Calendar calendar = Calendar.getInstance();
                        calendar.setTimeInMillis(taskData.getTimeStart());
                        int h = calendar.get(Calendar.HOUR_OF_DAY);
                        int m = calendar.get(Calendar.MINUTE);
                        calendar.clear();
                        calendar.set(i, i1, i2, h, m);
                        taskData.setTimeStart(calendar.getTimeInMillis());

                        //Update text
                        editDateStart.setText(Util.formatMillis(taskData.getTimeStart(), "dd/MM/yyyy"));

                        //Update timeDateEnd if necessary
                        if(taskData.getTimeEnd() < taskData.getTimeStart()){
                            taskData.setTimeEnd(taskData.getTimeStart() + 3600000);

                            //Update text
                            editDateEnd.setText(Util.formatMillis(taskData.getTimeEnd(), "dd/MM/yyyy"));
                            editTimeEnd.setText(Util.formatMillis(taskData.getTimeEnd(), "HH:mm"));
                        }
                    }
                }, calendar.get(Calendar.YEAR), calendar.get(Calendar.MONTH), calendar.get(Calendar.DAY_OF_MONTH))
                        .show();
            }
        });
        editDateEnd.setFocusable(false);
        editDateEnd.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                Calendar calendar = Calendar.getInstance();
                calendar.setTimeInMillis(taskData.getTimeEnd());
                new DatePickerDialog(view.getContext(), new DatePickerDialog.OnDateSetListener() {
                    @Override
                    public void onDateSet(DatePicker datePicker, int i, int i1, int i2) {
                        Calendar calendar = Calendar.getInstance();
                        calendar.setTimeInMillis(taskData.getTimeEnd());
                        int h = calendar.get(Calendar.HOUR_OF_DAY);
                        int m = calendar.get(Calendar.MINUTE);
                        calendar.clear();
                        calendar.set(i, i1, i2, h, m);
                        taskData.setTimeEnd(calendar.getTimeInMillis());

                        //Update text
                        editDateEnd.setText(Util.formatMillis(taskData.getTimeEnd(), "dd/MM/yyyy"));
                    }
                }, calendar.get(Calendar.YEAR), calendar.get(Calendar.MONTH), calendar.get(Calendar.DAY_OF_MONTH))
                        .show();
            }
        });
        editTimeStart.setFocusable(false);
        editTimeStart.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                Calendar calendar = Calendar.getInstance();
                calendar.setTimeInMillis(taskData.getTimeStart());
                new TimePickerDialog(view.getContext(), new TimePickerDialog.OnTimeSetListener() {
                    @Override
                    public void onTimeSet(TimePicker timePicker, int i, int i1) {
                        Calendar calendar = Calendar.getInstance();
                        calendar.setTimeInMillis(taskData.getTimeStart());
                        int y = calendar.get(Calendar.YEAR);
                        int m = calendar.get(Calendar.MONTH);
                        int d = calendar.get(Calendar.DAY_OF_MONTH);
                        calendar.clear();
                        calendar.set(y, m, d, i, i1);
                        taskData.setTimeStart(calendar.getTimeInMillis());

                        //Update text
                        editTimeStart.setText(Util.formatMillis(taskData.getTimeStart(), "HH:mm"));

                        //Check if needs to update timeEnd as well
                        if(taskData.getTimeEnd() < taskData.getTimeStart()){
                            taskData.setTimeEnd(taskData.getTimeStart() + 3600000);

                            //Update text
                            editDateEnd.setText(Util.formatMillis(taskData.getTimeEnd(), "dd/MM/yyyy"));
                            editTimeEnd.setText(Util.formatMillis(taskData.getTimeEnd(), "HH:mm"));
                        }
                    }
                }, calendar.get(Calendar.HOUR_OF_DAY), calendar.get(Calendar.MINUTE), true)
                        .show();
            }
        });
        editTimeEnd.setFocusable(false);
        editTimeEnd.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                Calendar calendar = Calendar.getInstance();
                calendar.setTimeInMillis(taskData.getTimeEnd());
                new TimePickerDialog(view.getContext(), new TimePickerDialog.OnTimeSetListener() {
                    @Override
                    public void onTimeSet(TimePicker timePicker, int i, int i1) {
                        Calendar calendar = Calendar.getInstance();
                        calendar.setTimeInMillis(taskData.getTimeEnd());
                        int y = calendar.get(Calendar.YEAR);
                        int m = calendar.get(Calendar.MONTH);
                        int d = calendar.get(Calendar.DAY_OF_MONTH);
                        calendar.clear();
                        calendar.set(y, m, d, i, i1);
                        taskData.setTimeEnd(calendar.getTimeInMillis());

                        //Update text
                        editTimeEnd.setText(Util.formatMillis(taskData.getTimeEnd(), "HH:mm"));
                    }
                }, calendar.get(Calendar.HOUR_OF_DAY), calendar.get(Calendar.MINUTE), true)
                        .show();
            }
        });
        editNotify.setOnEditorActionListener(new TextView.OnEditorActionListener() {
            @Override
            public boolean onEditorAction(TextView textView, int i, KeyEvent keyEvent) {
                if(i == EditorInfo.IME_ACTION_DONE) {
                    InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
                    imm.hideSoftInputFromWindow(editNotify.getWindowToken(), 0);
                    editNotify.clearFocus();
                    return true;
                }
                return false;
            }
        });
        switchRepeat.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton compoundButton, boolean b) {
                editRepeatEvery.setEnabled(b);
                editRepeatTimes.setEnabled(b);
                labelRepeatEvery.setEnabled(b);
                labelRepeatTimes.setEnabled(b);
            }
        });


        //Set data
        editTitle.setText(taskData.getTitle());
        editDescription.setText(taskData.getDescription());
        editDateStart.setText(Util.formatMillis(taskData.getTimeStart(), "dd/MM/yyyy"));
        editDateEnd.setText(Util.formatMillis(taskData.getTimeEnd(), "dd/MM/yyyy"));
        editTimeStart.setText(Util.formatMillis(taskData.getTimeStart(), "HH:mm"));
        editTimeEnd.setText(Util.formatMillis(taskData.getTimeEnd(), "HH:mm"));
        editNotify.setText(taskData.getTimeNotify() == 0?"":""+(taskData.getTimeStart()-taskData.getTimeNotify())/60000);
        if(taskData.getTaskID() == 0) {//Can only set repeat if creating new task!
            switchRepeat.setChecked(false);
            editRepeatEvery.setEnabled(false);
            editRepeatTimes.setEnabled(false);
            labelRepeatEvery.setEnabled(false);
            labelRepeatTimes.setEnabled(false);
        }
        else{
            switchRepeat.setVisibility(View.GONE);
            editRepeatEvery.setVisibility(View.GONE);
            editRepeatTimes.setVisibility(View.GONE);
            labelRepeatEvery.setVisibility(View.GONE);
            labelRepeatTimes.setVisibility(View.GONE);
            labelRepeat.setVisibility(View.GONE);
        }

    }


    private void checkAndSave(){
        System.out.println("Check and save!");
        //Reset all error in the views
        editTitle.setError(null);
        editDescription.setError(null);
        editDateEnd.setError(null);
        editTimeEnd.setError(null);
        editNotify.setError(null);
        editRepeatEvery.setError(null);
        editRepeatTimes.setError(null);

        //Check if all entries are valid
        boolean valid = true;

        //Check title
        taskData.setTitle(""+editTitle.getText());
        if(taskData.getTitle().equals("") || taskData.getTitle().equals(" ")){
            editTitle.setError("Required!");
            valid = false;
        }
        else if(taskData.getTitle().contains(";")){
            editTitle.setError("Can't contain ';'!");
            valid = false;
        }

        //Check description
        taskData.setDescription(""+editDescription.getText());
        if(taskData.getDescription().contains(";")){
            editDescription.setError("Can't contain ';'!");
            valid = false;
        }

        //Check date and time
        if(taskData.getTimeStart() >= taskData.getTimeEnd()){
            editDateEnd.setError("Invalid!");
            editTimeEnd.setError("Invalid!");
            valid = false;
        }

        //Check notify time
        if((""+editNotify.getText()).equals("")) taskData.setTimeNotify(0);
        else {
            try {
                int minutes = Integer.parseInt("" + editNotify.getText());

                if(minutes < 0){
                    editNotify.setError("Must be positive or empty!");
                    valid = false;
                }
                else{
                    // When valid minutes -> calculate and save notifyTime
                    Calendar calendar = Calendar.getInstance();
                    calendar.setTimeInMillis(taskData.getTimeStart());
                    calendar.add(Calendar.MINUTE, -1*minutes);
                    taskData.setTimeNotify(calendar.getTimeInMillis());
                }

            } catch (Exception e) {
                editNotify.setError("Enter a number or leave empty!");
                valid = false;
            }
        }

        //Check repeat stuff
        if(switchRepeat.isChecked()){
            try{
                int repeatEvery = Integer.parseInt(""+editRepeatEvery.getText());
                if(repeatEvery <= 0){
                    editRepeatEvery.setError("Must be > zero!");
                    valid = false;
                }
            }catch (Exception e){
                editRepeatEvery.setError("Enter a number!");
                valid = false;
            }
            try{
                int repeatTimes = Integer.parseInt(""+editRepeatTimes.getText());
                if(repeatTimes <= 1){
                    editRepeatTimes.setError("Must be > one!");
                    valid = false;
                }
            }catch (Exception e){
                editRepeatTimes.setError("Enter a number!");
                valid = false;
            }
        }


        //Return if not valid
        if(!valid) return;



        //Save task
        if(!switchRepeat.isChecked()) {//When no repeat
            boolean newTask = taskData.getTaskID() == 0;
            if (newTask) taskData.setTaskID(taskUtil.getNextTaskID());

            taskUtil.saveTask(taskData);
        }
        else{
            //When repeat
            int repeatEvery = Integer.parseInt(""+editRepeatEvery.getText());
            int repeatTimes = Integer.parseInt(""+editRepeatTimes.getText());

            long firstTimeStart = taskData.getTimeStart();
            long firstTimeEnd = taskData.getTimeEnd();
            long firstTimeNotify = taskData.getTimeNotify();
            boolean notify = firstTimeNotify != 0;

            Calendar calendarStart = Calendar.getInstance();
            Calendar calendarEnd = Calendar.getInstance();
            Calendar calendarNotify = Calendar.getInstance();
            calendarStart.setTimeInMillis(firstTimeStart);
            calendarEnd.setTimeInMillis(firstTimeEnd);
            calendarNotify.setTimeInMillis(firstTimeNotify);

            taskData.setBatchID(taskUtil.getNextBatchID());
            for(int i=0; i<repeatTimes; i++){
                taskData.setTaskID(taskUtil.getNextTaskID());
                System.out.println(taskData.getTaskID());

                taskData.setTimeStart(calendarStart.getTimeInMillis());
                taskData.setTimeEnd(calendarEnd.getTimeInMillis());
                taskData.setTimeNotify(notify ? calendarNotify.getTimeInMillis() : 0);

                taskUtil.saveTask(taskData);

                calendarStart.add(Calendar.DATE, repeatEvery);
                calendarEnd.add(Calendar.DATE, repeatEvery);
                calendarNotify.add(Calendar.DATE, repeatEvery);
            }
        }

        //Exit activity
        finish();
    }


    @Override
    public boolean onCreateOptionsMenu(final Menu menu) {
        getMenuInflater().inflate(R.menu.menu_edit, menu);

        //When creating new task -> can't remove it yet
        if(taskData.getTaskID() == 0) {
            menu.removeItem(R.id.edit_menu_remove);
        }

        //When creating new task or editing task not from batch -> can't remove batch
        if(taskData.getTaskID() == 0 || taskData.getBatchID() == -1){
            menu.removeItem(R.id.edit_menu_remove_batch);
        }

        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case R.id.edit_menu_remove:
                //Show alert dialog
                AlertDialog.Builder builder = new AlertDialog.Builder(this);
                builder.setTitle("Are you sure?");
                builder.setMessage("You're about to remove this appointment.");
                builder.setPositiveButton("Remove", new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialogInterface, int i) {
                        taskUtil.removeTask(taskData.getTaskID());
                        finish();
                    }
                });
                builder.setNegativeButton("Cancel", null);
                AlertDialog dialog = builder.create();
                dialog.show();
                return true;

            case R.id.edit_menu_remove_batch:
                //Show alert dialog
                builder = new AlertDialog.Builder(this);
                builder.setTitle("Are you sure?");
                builder.setMessage("You're about to remove this entire batch of appointments.");
                builder.setPositiveButton("Remove", new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialogInterface, int i) {
                        //Remove batch
                        TaskData[] rem = taskUtil.getByBatchID(taskData.getBatchID());
                        for(int j=0; j<rem.length; j++) {
                            taskUtil.removeTask(rem[j].getTaskID());
                        }
                        finish();
                    }
                });
                builder.setNegativeButton("Cancel", null);
                dialog = builder.create();
                dialog.show();
                return true;

            case R.id.edit_menu_save:
                checkAndSave();
                return true;

            case android.R.id.home:
                onBackPressed();
                return true;

            default:
                return super.onOptionsItemSelected(item);
        }
    }

    @Override
    public void onBackPressed() {
        //Close activity
        this.finish();
    }

}
