/*
 * Copyright (C) 2021 Thomas Van Acker
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

package be.bitscuit.calendar.BroadcastReceivers;

import android.app.AlarmManager;
import android.app.PendingIntent;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;

import be.bitscuit.calendar.Util.TaskData;
import be.bitscuit.calendar.Util.TaskUtil;
import be.bitscuit.calendar.Util.UserPrefs;
import be.bitscuit.calendar.Util.Util;

import static android.content.Context.ALARM_SERVICE;

public class BootReceiver extends BroadcastReceiver {

    @Override
    public void onReceive(Context context, Intent intent) {

        //Set all the alarms, they were removed due to reboot
        TaskUtil taskUtil = new TaskUtil(context);
        TaskData[] tasks = taskUtil.getAllTasks();

        for(int i=0; i<tasks.length; i++){
            if(tasks[i].getTimeStart() > System.currentTimeMillis()){//Only set alarm if in future
                taskUtil.setAlarmForTask(tasks[i]);
            }
        }


        //Set new alarm for overview
        UserPrefs userPrefs = new UserPrefs(context);
        if(userPrefs.getSendDailyOverview()) {
            AlarmManager alarmManager = (AlarmManager) context.getSystemService(ALARM_SERVICE);
            Intent i = new Intent(context.getApplicationContext(), OverviewReceiver.class);
            PendingIntent pendingIntent = PendingIntent.getBroadcast(context.getApplicationContext(), 0, i, 0);
            long time = Util.millisToDateMillis(System.currentTimeMillis()) + userPrefs.getDailyOverviewTime();
            if (time < System.currentTimeMillis()) time += 86400000; //When next overview is tomorrow
            alarmManager.setRepeating(AlarmManager.RTC_WAKEUP, time, 86400000, pendingIntent);
        }

        //Set new alarm for sleep reminder
        if(userPrefs.getSendSleepReminder()) {
            AlarmManager alarmManager = (AlarmManager) context.getSystemService(ALARM_SERVICE);
            Intent i = new Intent(context.getApplicationContext(), SleepReceiver.class);
            PendingIntent pendingIntent = PendingIntent.getBroadcast(context.getApplicationContext(), 0, i, 0);
            long time = Util.millisToDateMillis(System.currentTimeMillis()) + userPrefs.getSleepTime();
            if (time < System.currentTimeMillis()) time += 86400000; //When next overview is tomorrow
            alarmManager.setRepeating(AlarmManager.RTC_WAKEUP, time, 86400000, pendingIntent);
        }


    }
}
