/*
 * Copyright (C) 2021 Thomas Van Acker
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

package be.bitscuit.calendar.Util;

import android.content.Context;
import android.content.SharedPreferences;

public class UserPrefs {
    
    private Context context;
    private SharedPreferences prefs;
    private SharedPreferences.Editor editor;
    
    public UserPrefs(Context context){
        this.context = context;
        prefs = context.getSharedPreferences("USER_PRES", Context.MODE_PRIVATE);
        editor = prefs.edit();
    }
    
    
    public boolean getAutoDeleteTasks(){
        return prefs.getBoolean("AUTO_DELETE_TASKS", false);
    }
    
    public void setAutoDeleteTasks(boolean b){
        editor.putBoolean("AUTO_DELETE_TASKS", b);
        editor.commit();
    }
    
    public long getAutoDeleteTasksAfter(){
        return prefs.getLong("AUTO_DELETE_TASKS_AFTER", 86400000*14); 
    }
    
    public void setAutoDeleteTasksAfter(long after){
        editor.putLong("AUTO_DELETE_TASKS_AFTER", after);
        editor.commit();
    }

    public boolean getSendDailyOverview(){
        return prefs.getBoolean("SEND_DAILY_OVERVIEW", false);
    }

    public void setSendDailyOverview(boolean b){
        editor.putBoolean("SEND_DAILY_OVERVIEW", b);
        editor.commit();
    }

    public long getDailyOverviewTime(){
        return prefs.getLong("DAILY_OVERVIEW_TIME", 3600000*7);
    }

    public void setDailyOverviewTime(long time){
        editor.putLong("DAILY_OVERVIEW_TIME", time);
        editor.commit();
    }

    public boolean getSendSleepReminder(){
        return prefs.getBoolean("SEND_SLEEP_REMINDER", false);
    }

    public void setSendSleepReminder(boolean b){
        editor.putBoolean("SEND_SLEEP_REMINDER", b);
        editor.commit();
    }

    public long getSleepTime(){
        return prefs.getLong("SLEEP_TIME", 3600000*22);
    }

    public void setSleepTime(long time){
        editor.putLong("SLEEP_TIME", time);
        editor.commit();
    }
    
}
